/// <reference types="node" />
import { LunarClientMod, LunarClientPluginChannel, NotificationLevel, ServerRuleEnum, ServerRuleValue, StaffMod, TeammatePlayer, Waypoint } from './constants';
import type { ModSettingsPacket } from './protocol';
/**
 * Class that represents a Lunar Client player
 */
export declare class LunarClientPlayer {
    /**
     * Whether or not a connection
     * has been established with the
     * client
     */
    isConnected: boolean;
    /** UUID for this player */
    uuid: string;
    /**
     * Plugin channel used to establish the
     * connection, defaults to
     * `LunarClientPluginChannel.NEW`
     */
    readonly channel: LunarClientPluginChannel;
    /** Waypoints loaded by the client */
    waypoints: Waypoint[];
    /** Current teammates of the client */
    teammates: TeammatePlayer[];
    /**
     * Current active cooldowns.
     * This list contains the ids of the cooldowns
     */
    cooldowns: string[];
    /**
     * Current mod settings for the client. Key (mod id): Value (mod setting)
     */
    modSettings: ModSettingsPacket['data']['settings'];
    /** Options passed in the constructor */
    private readonly options?;
    /** Internal packet handler */
    private readonly handler;
    /**
     * Instantiate a new Lunar Client player
     * class.
     *
     * This high level API will let you
     * *"control"* a bit the client.
     * @param options Options to pass
     * @example
     * ```javascript
     * const player = new LunarClientPlayer({
     *   customHandling: {
     *     sendPacket: (buffer) {
     *        // Somehow send the packet
     *        // to the client
     *     },
     *     registerPluginChannel: (channel) {
     *        // Somehow register the channel
     *     }
     *   }
     * });
     *
     * // Establish the connection
     * // using plugin channels
     * player.connect();
     *
     * // You can now call any method
     * // from the class
     * ```
     */
    constructor(options?: LunarClientPlayerOptions);
    /**
     * Send the register
     * plugin channel packet
     */
    connect(): void;
    /**
     * Add a waypoint to the client
     * @param waypoint Waypoint to add
     * @returns Whether or not this operation was successful
     */
    addWaypoint(waypoint: Waypoint): boolean;
    /**
     * Remove a waypoint from the client
     * @param waypoint Waypoint object or waypoint name
     * @returns Whether or not this operation was successful
     */
    removeWaypoint(waypoint: string | Waypoint): boolean;
    /**
     * Remove all waypoints loaded by the player
     */
    removeAllWaypoints(): void;
    /** Send a notification to the client
     * @param message Message to send
     * @param durationMs Duration of the message in milliseconds
     * @param level Message level, defaults to `NotificationLevel.INFO`
     * @deprecated This packet is not supported by the client by default
     */
    sendNotification(message: string, durationMs: number, level?: NotificationLevel): void;
    /**
     * Add a teammate to the client.
     *
     * The TeamView mod must be enabled
     * in order for this to work
     * @param teammate Teammate to add
     * @returns Whether or not this operation was successful
     */
    addTeammate(teammate: TeammatePlayer): boolean;
    /**
     * Remove a teammate from the client.
     *
     * The TeamView mod must be enabled
     * in order for this to work
     * @param teammate Teammate UUID or teammate object
     * @returns Whether or not this operation was successful
     */
    removeTeammate(teammate: string | TeammatePlayer): boolean;
    /**
     * Remove all teammates from the client.
     *
     * The TeamView mod must be enabled
     * in order for this to work
     */
    removeAllTeammates(): void;
    /**
     * Send the teammates packet
     * to the client
     */
    private sendTeammateList;
    /**
     * Add a cooldown to the client
     * @param id String id of the cooldown, used to remove it later
     * @param durationMs Duration of the cooldown in milliseconds
     * @param iconId Icon id to use, same system as [minecraft ids](https://minecraft-ids.grahamedgecombe.com/)
     * @returns Whether or not this operation was successful
     */
    addCooldown(id: string, durationMs: number, iconId: number): boolean;
    /**
     * Remove a cooldown from the client
     * @param id String id of the cooldown
     * @returns Whether or not this operation was successful
     */
    removeCooldown(id: string): boolean;
    /**
     * Set the state of the given Staff Mod for the client
     * @param mod Staff mod to apply the state to
     * @param state State to apply
     */
    setStaffModState(mod: StaffMod, state: boolean): void;
    /**
     * Set a server rule for the client.
     * @param serverRule Server rule to set the value to
     * @param value Value to set, usually a boolean but could be `NEUTRAL` or `FORCED_OFF` when the server rule is minimap status or number
     * @param valueType Tell the method how the value should be encoded, defaults to `boolean`
     */
    setServerRule(serverRule: ServerRuleEnum, value: boolean | number | string, valueType?: ServerRuleValue): void;
    /**
     * Set multiple rules at once
     * @see `LunarClientPlayer#setServerRule` for implementation
     * @param serverRules Rules to set
     */
    setServerRules(...serverRules: {
        serverRule: ServerRuleEnum;
        value: boolean | number | string;
        valueType?: ServerRuleValue;
    }[]): void;
    /**
     * Set a forced state for the given mod
     * @param mod Mod to set the setting for
     * @param enabled Whether the mod should be force enabled or force disabled
     * @param properties
     * @returns Whether or not the operation was successful
     */
    addModSetting(mod: LunarClientMod, enabled: boolean, properties: {
        [key: string]: unknown;
    }): boolean;
    /**
     * Remove a forced state for the given mod
     * @param mod Mod to remove the forced state
     * @returns Whether or not the operation was successful
     */
    removeModSetting(mod: LunarClientMod): boolean;
    /**
     * Send the ModSettings
     * packet to the client
     */
    private sendModSettings;
    /**
     * Handle the given packet.
     *
     * Call this method only if you
     * want to manually override
     * the packet handling
     * @param buffer Packet to handle
     */
    handlePacket(buffer: Buffer): void;
    /**
     * Send a packet to the client, will use
     * the provided sendPacket function in options
     * if the protocol handler isn't available.
     * @param buffer Packet to send to the client
     */
    private sendPacket;
}
/**
 * Options that can be passed to
 * instantiate a new LunarClientPlayer
 */
export interface LunarClientPlayerOptions {
    /**
     * Whether or not the plugin channel
     * has already been registered.
     *
     * If this is set to `true`, the channel
     * registration process will be skipped
     * and this instance will asume
     * that the channel is already registedred
     * and that the player is running
     * Lunar Client.
     */
    channelAlreadyRegistered?: boolean;
    /**
     * Plugin channel to use in the
     * channel registration process
     */
    pluginChannel?: LunarClientPluginChannel;
    /**
     * Functions that will be called when
     * a certain action should be done,
     * for example, sending a packet.
     *
     * For now you must specify these
     * because no protocol handler
     * is written yet. If you don't specify
     * any an error will be thrown.
     */
    customHandling: {
        sendPacket: (buffer: Buffer) => void;
        registerPluginChannel: (channel: LunarClientPluginChannel) => void;
    };
    /**
     * UUID for this player
     *
     * This is used in the teammates
     * packet for the leader field
     * @see [TeammatesPacket](./protocol/client/TeammatesPacket.ts)
     */
    playerUUID?: string;
}
