"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LunarClientPlayer = void 0;
const constants_1 = require("./constants");
const Packet_1 = require("./protocol/Packet");
const PacketBuilder_1 = require("./protocol/PacketBuilder");
/**
 * Class that represents a Lunar Client player
 */
class LunarClientPlayer {
    /**
     * Instantiate a new Lunar Client player
     * class.
     *
     * This high level API will let you
     * *"control"* a bit the client.
     * @param options Options to pass
     * @example
     * ```javascript
     * const player = new LunarClientPlayer({
     *   customHandling: {
     *     sendPacket: (buffer) {
     *        // Somehow send the packet
     *        // to the client
     *     },
     *     registerPluginChannel: (channel) {
     *        // Somehow register the channel
     *     }
     *   }
     * });
     *
     * // Establish the connection
     * // using plugin channels
     * player.connect();
     *
     * // You can now call any method
     * // from the class
     * ```
     */
    constructor(options) {
        this.options = options;
        if (this.options?.channelAlreadyRegistered)
            this.isConnected = true;
        else
            this.isConnected = false;
        this.uuid = options?.playerUUID ?? '';
        this.channel = this.options?.pluginChannel || constants_1.LunarClientPluginChannel.NEW;
        this.waypoints = [];
        this.teammates = [];
        this.cooldowns = [];
    }
    /**
     * Send the register
     * plugin channel packet
     */
    connect() {
        if (this.options?.customHandling?.registerPluginChannel) {
            this.options.customHandling.registerPluginChannel(this.channel);
            this.isConnected = true;
            return;
        }
        throw new Error('For now, you must provide a registerPluginChannel function in the options');
    }
    /**
     * Add a waypoint to the client
     * @param waypoint Waypoint to add
     * @returns Whether or not this operation was successful
     */
    addWaypoint(waypoint) {
        if (this.waypoints.find((w) => w.name === waypoint.name))
            return false;
        this.waypoints.push(waypoint);
        const { packet } = new PacketBuilder_1.default('WaypointAddPacket', {
            ...waypoint,
            world: '',
        });
        this.sendPacket(packet);
        return true;
    }
    /**
     * Remove a waypoint from the client
     * @param waypoint Waypoint object or waypoint name
     * @returns Whether or not this operation was successful
     */
    removeWaypoint(waypoint) {
        waypoint = typeof waypoint === 'string' ? waypoint : waypoint.name;
        if (!this.waypoints.find((w) => w.name === waypoint))
            return false;
        this.waypoints = this.waypoints.filter((w) => w.name !== waypoint);
        const { packet } = new PacketBuilder_1.default('WaypointRemovePacket', {
            name: waypoint,
            world: '',
        });
        this.sendPacket(packet);
        return true;
    }
    /**
     * Remove all waypoints loaded by the player
     */
    removeAllWaypoints() {
        for (const waypoint of this.waypoints)
            this.removeWaypoint(waypoint);
        this.waypoints = [];
    }
    /** Send a notification to the client
     * @param message Message to send
     * @param durationMs Duration of the message in milliseconds
     * @param level Message level, defaults to `NotificationLevel.INFO`
     * @deprecated This packet is not supported by the client by default
     */
    sendNotification(message, durationMs, level = constants_1.NotificationLevel.INFO) {
        const { packet } = new PacketBuilder_1.default('NotificationPacket', {
            message,
            durationMs,
            level,
        });
        this.sendPacket(packet);
    }
    /**
     * Add a teammate to the client.
     *
     * The TeamView mod must be enabled
     * in order for this to work
     * @param teammate Teammate to add
     * @returns Whether or not this operation was successful
     */
    addTeammate(teammate) {
        if (this.teammates.find((t) => t.uuid === teammate.uuid))
            return false;
        this.teammates.push(teammate);
        this.sendTeammateList();
        return true;
    }
    /**
     * Remove a teammate from the client.
     *
     * The TeamView mod must be enabled
     * in order for this to work
     * @param teammate Teammate UUID or teammate object
     * @returns Whether or not this operation was successful
     */
    removeTeammate(teammate) {
        teammate = typeof teammate === 'string' ? teammate : teammate.uuid;
        if (!this.teammates.find((t) => t.uuid === teammate))
            return false;
        this.teammates = this.teammates.filter((t) => t.uuid !== teammate);
        this.sendTeammateList();
        return true;
    }
    /**
     * Remove all teammates from the client.
     *
     * The TeamView mod must be enabled
     * in order for this to work
     */
    removeAllTeammates() {
        this.teammates = [];
        this.sendTeammateList();
    }
    /**
     * Send the teammates packet
     * to the client
     */
    sendTeammateList() {
        const { packet } = new PacketBuilder_1.default('TeammatesPacket', {
            leader: this.uuid,
            lastMs: Date.now(),
            players: this.teammates.map((teammate) => ({
                uuid: teammate.uuid,
                posMap: {
                    x: teammate.x ?? 0,
                    y: teammate.y ?? 0,
                    z: teammate.z ?? 0,
                },
            })),
        });
        this.sendPacket(packet);
    }
    /**
     * Add a cooldown to the client
     * @param id String id of the cooldown, used to remove it later
     * @param durationMs Duration of the cooldown in milliseconds
     * @param iconId Icon id to use, same system as [minecraft ids](https://minecraft-ids.grahamedgecombe.com/)
     * @returns Whether or not this operation was successful
     */
    addCooldown(id, durationMs, iconId) {
        if (this.cooldowns.find((c) => c === id))
            return false;
        this.cooldowns.push(id);
        setTimeout(() => (this.cooldowns = this.cooldowns.filter((c) => c !== id)), durationMs);
        const { packet } = new PacketBuilder_1.default('CooldownPacket', {
            id,
            durationMs,
            iconId,
        });
        this.sendPacket(packet);
        return true;
    }
    /**
     * Remove a cooldown from the client
     * @param id String id of the cooldown
     * @returns Whether or not this operation was successful
     */
    removeCooldown(id) {
        if (!this.cooldowns.find((c) => c === id))
            return false;
        this.cooldowns = this.cooldowns.filter((c) => c !== id);
        const { packet } = new PacketBuilder_1.default('CooldownPacket', {
            id,
            durationMs: 0,
            iconId: 0,
        });
        this.sendPacket(packet);
        return true;
    }
    /**
     * Set the state of the given Staff Mod for the client
     * @param mod Staff mod to apply the state to
     * @param state State to apply
     */
    setStaffModState(mod, state) {
        const { packet } = new PacketBuilder_1.default('StaffModStatePacket', { mod, state });
        this.sendPacket(packet);
    }
    /**
     * Set a server rule for the client.
     * @param serverRule Server rule to set the value to
     * @param value Value to set, usually a boolean but could be `NEUTRAL` or `FORCED_OFF` when the server rule is minimap status or number
     * @param valueType Tell the method how the value should be encoded, defaults to `boolean`
     */
    setServerRule(serverRule, value, valueType = 'boolean') {
        // Known server rule that has a string
        if (serverRule === constants_1.ServerRuleEnum.MINIMAP_STATUS)
            valueType = 'string';
        const data = { rule: serverRule };
        // @ts-ignore I have no idea on how
        // to type this but it works fine
        data[`${valueType}Value`] = value;
        const { packet } = new PacketBuilder_1.default('ServerRulePacket', data);
        this.sendPacket(packet);
    }
    /**
     * Set multiple rules at once
     * @see `LunarClientPlayer#setServerRule` for implementation
     * @param serverRules Rules to set
     */
    setServerRules(...serverRules) {
        for (const rule of serverRules)
            this.setServerRule(rule.serverRule, rule.value, rule.valueType ?? 'boolean');
    }
    /**
     * Set a forced state for the given mod
     * @param mod Mod to set the setting for
     * @param enabled Whether the mod should be force enabled or force disabled
     * @param properties
     * @returns Whether or not the operation was successful
     */
    addModSetting(mod, enabled, properties) {
        if (this.modSettings[mod])
            return false;
        this.modSettings[mod] = { enabled, properties };
        this.sendModSettings();
        return true;
    }
    /**
     * Remove a forced state for the given mod
     * @param mod Mod to remove the forced state
     * @returns Whether or not the operation was successful
     */
    removeModSetting(mod) {
        if (!this.modSettings[mod])
            return false;
        delete this.modSettings[mod];
        this.sendModSettings();
        return true;
    }
    /**
     * Send the ModSettings
     * packet to the client
     */
    sendModSettings() {
        const { packet } = new PacketBuilder_1.default('ModSettingsPacket', {
            settings: this.modSettings,
        });
        this.sendPacket(packet);
    }
    /**
     * Handle the given packet.
     *
     * Call this method only if you
     * want to manually override
     * the packet handling
     * @param buffer Packet to handle
     */
    handlePacket(buffer) {
        this.handler.handle(buffer);
    }
    /**
     * Send a packet to the client, will use
     * the provided sendPacket function in options
     * if the protocol handler isn't available.
     * @param buffer Packet to send to the client
     */
    sendPacket(packet) {
        const buffer = packet instanceof Packet_1.Packet ? packet.buf.buffer : packet;
        if (this.options?.customHandling?.sendPacket)
            this.options.customHandling.sendPacket(buffer);
        throw new Error('For now, you must provide a sendPacket function in the options');
    }
}
exports.LunarClientPlayer = LunarClientPlayer;
